/* AbulEduCameraControllerV1
 *
 * @warning aucun traitement d'erreur n'est pour l'instant implémenté
 * @see https://redmine.ryxeo.com/projects/leterrier-developpeu/repository
 * @author 2015 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
 * @see The GNU Public License (GPL)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef ABULEDUCAMERACONTROLLERV1_H
#define ABULEDUCAMERACONTROLLERV1_H

#include <QObject>
#include <QDebug>
#include <QThread>
#include <QTime>
#include <QQueue>
#include <QFlags>

#include "abuleducameraworkerv1.h"

#include "config.h"

///
/// \brief AbulEduCameraV1 est une classe permettant de connecter une caméra et de récupérer les images capturées
///
/// C'est une classe facilitant l'accès à la classe AbulEduCameraWorker.
///
/// \code
/// A utiliser dans la classe appelante
///    if(!m_camera)
///    {
///       m_camera = new AbulEduCameraV1(this);
///       connect(m_camera, SIGNAL(signalFullImageReady(QImage)), this, SLOT(slotGetQImageFromCam(QImage)));
///       m_camera->abeCameraInitCamera(0);
///       m_camera->abeCameraStartCapture();
///    }

//qRegisterMetaType< cv::Mat >("cv::Mat");

class AbulEduCameraControllerV1 : public QObject
{
    Q_OBJECT



public:
    enum Control {
        NoOptions   = 0x0,
        ImageNeeded = 0x1, // Une image est demandée
        ImageSend   = 0x2, // Une image a été envoyée
        Ready = ImageNeeded | ImageSend // Pret à demander une image à la caméra, on attend le timeout
    };
    Q_DECLARE_FLAGS(Controls, Control)
    Q_FLAGS(Control Controls)


    enum Image
    {
        NoFlag          = 0x0,
        FullQImage      = 0x1, // QImage  complète
        FullCvMat       = 0x2, // cv::Mat complète
        CroppedImage    = 0x4, // QImage  zone sélectionnée ROI
        CroppedCvMat    = 0x8  // cv::Mat zone sélectionnée ROI
    };
    Q_DECLARE_FLAGS(Images, Image)

    explicit AbulEduCameraControllerV1(QObject *parent = 0);
    ~AbulEduCameraControllerV1();

    ///
    /// \brief abeCameraInitCamera ouvre la caméra avec les paramètres fournis
    /// \param deviceNumber le numero de la caméra
    /// \param resWidth la résolution souhaitée (largeur en px)
    /// \param resHeight la résolution souhaitée (hauteur en px)
    /// \note les paramètres de résolution peuvent ne pas être atteints en fonction des possibilités de la caméra
    /// la résolution de la caméra sera celle immédiatement inférieure aux paramètres passés
    ///
    void abeCameraInitCamera(int deviceNumber= 0, int resWidth = DEFAULT_HORIZONTAL_RES, int resHeight = DEFAULT_VERTICAL_RES);
    
    ///
    /// \brief abeCameraSetFrameResolution fixe la résolution de l'image en pixels.
    /// \param width en pixels
    /// \param height en pixels
    /// \note les paramètres de résolution peuvent ne pas être atteints en fonction des possibilités de la caméra
    /// la résolution de la caméra sera celle immédiatement inférieure aux paramètres passés
    ///
    void abeCameraSetFrameResolution(int width, int height);

    ///
    /// \brief abeCameraSetImagesTypes précise le type d'images que l'on veut capturer
    /// \param imageType voir enum Image
    ///
    void abeCameraSetImagesTypes(Images imageType);

    ///
    /// \brief abeCameraSetFrameRefreshInterval fixe le delai (en ms) entre deux captures.
    /// \param refreshInterval nombre de millisecondes entre deux captures (doit être supérieur ou égal à 0)
    ///
    void abeCameraSetFrameRefreshInterval(int refreshInterval = DEFAULT_REFRESH_INTERVAL);

    ///
    /// \brief abeCameraSetROI Zone sélectionnée dans l'image capturée
    /// \param x
    /// \param y
    /// \param width
    /// \param height
    ///
    void abeCameraSetROI(int x, int y, int width, int height);

    ///
    /// \brief abeCameraSetROI Zone sélectionnée dans l'image capturée
    /// \param rect Le Qrect de la zone sélectionnée dans l'image
    ///
    void abeCameraSetROI(QRect rect);

    ///
    /// \brief abeCameraStart déclenche la capture des images.
    ///
    /// le départ de la capture peut être différé de x ms.
    /// \param delai valeur en ms avant que la caméra commence à capturer.
    ///
    void abeCameraStartCapture(int delai = 0);

    ///
    /// \brief abeCameraStop arrête la capture des images.
    ///
    /// L'arrêt peut être différé de x ms.
    /// \param delai valeur en ms avant que la caméra arrête de capturer.
    ///
    void abeCameraStopCapture(int delai = 0);

private:

    ///
    /// \brief updateFPS Calcule le FPS moyen sur CAPTURE_FPS_STAT_QUEUE_LENGTH frames (32 par défaut)
    /// \param timeElapsed Le temps mis pour capturer une image.
    ///
    void updateFPS(int timeElapsed);

    ///
    /// \brief createCameraThread Place le cameraWorker dans un thread séparé
    ///
    void createCameraThread();

    bool m_localDebug;
    AbulEduCameraWorker *m_camWorker;
    QThread             *m_camThread;
    QTimer              *m_camTimer;
    QTime               *m_globalTime;
    QQueue<int>  m_fpsQueue;
    int m_elapsedGlobalTime;
    int sampleNumber;
    int fpsSum;
    int m_deviceNumber;
    int m_resWidth;
    int m_resHeight;
    int m_frameRefreshInterval;
    int m_imagesCount;
    Controls m_flagReady;
    AbulEduCameraWorker::ImageTypes m_flagImageTypes;


public slots:
    ///
    /// \brief slotQImageFromCamReceived On a reçu une image de la camera
    /// \param image La QImage reçue
    ///
    void slotQImageFromCamReceived(QImage image);

    ///
    /// \brief slotCvMatFromCamReceived On a reçu une image de la camera
    /// \param image La cv::Mat reçue
    ///
    void slotCvMatFromCamReceived(cv::Mat frame);

    ///
    /// \brief slotGetImageSize Récupère la taille de l'image capturée (pas implémenté)
    /// \param width
    /// \param height
    ///
    void slotGetImageSize(int width, int height);

    ///
    /// \brief abeCameraGetFrame déclenche la capture d'une image
    /// \note le type d'images capturées est déterminé par abeCameraSetImagesTypes(Images imageType);
    ///
    void abeCameraGetFrame();

    ///
    /// \brief abeCameraGetFullCvMatImage
    /// \obsolete
    void abeCameraGetFullCvMatImage();
    ///
    /// \brief abeCameraGetCroppedImage
    /// \obsolete
    void abeCameraGetCroppedImage();
    ///
    /// \brief abeCameraGetCroppedCvMatImage
    /// \obsolete
    void abeCameraGetCroppedCvMatImage();

    ///
    /// \brief slotCameraWorkerErrorHandler traite les erreurs retournée par le CameraWorker
    /// \param errorNumber
    ///
    void slotCameraWorkerErrorHandler(int errorNumber);

    ///
    /// \brief slotCameraControllerReadyToGetNewImage signifie au Camara controller que l'on est pret à capturer une image (ou pas)
    /// \param ready true, on peut demander une image au CaméraController, false on n'est pas pret à capturer une image
    ///
    void slotCameraControllerReadyToGetNewImage(bool ready);

signals:
    /* Signaux à destination du CameraWorker placé dans un autre thread */
    void signalSetDevice(int deviceNumber, int width = 9999, int height = 9999);
    void signalSetImageResolution(int, int);
    void signalGetFrame(AbulEduCameraWorker::ImageTypes);
    void signalGetFullImage();
    void signalGetFullCvMatImage();
    void signalGetCroppedImage();
    void signalGetCroppedCvMatImage();
    void signalSetROI(QRect);

    /* Signaux à destination de l'extérieur */
    void signalAbeCameraError(int errorNumber);
    void signalAbeCameraImageSize(int width, int height);
    void signalFullImageReady(QImage);
    void signalFullCvMatImageReady(cv::Mat);
    void signalCroppedImageReady(QImage);
    void signalCroppedCvMatImageReady(cv::Mat);
    void signalCameraFPSAverage(int fps);

};
    Q_DECLARE_OPERATORS_FOR_FLAGS(AbulEduCameraControllerV1::Controls)
    Q_DECLARE_OPERATORS_FOR_FLAGS(AbulEduCameraControllerV1::Images)

#endif // ABULEDUCAMERACONTROLLERV1_H
