/* AbulEduCameraWorkerV1
 *
 * @warning aucun traitement d'erreur n'est pour l'instant implémenté
 * @see https://redmine.ryxeo.com/projects/leterrier-developpeu/repository
 * @author 2015 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
 * @see The GNU Public License (GPL)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef ABULEDUCAMERAWORKER_H
#define ABULEDUCAMERAWORKER_H

#include <QObject>
//#include <QSize>
#include <QImage>
#include <QTimer>
#include <QTime>
#include <QDebug>

// opencv
#ifdef WITH_OPENCV2
  #include <opencv2/core/core.hpp>
  #include <opencv2/highgui/highgui.hpp>
  #include <opencv2/imgproc/imgproc.hpp>
#endif

///
/// \brief The AbulEduCameraWorker manages webcams
///
/// Cette classe gère l'acquisition d'images par le biais d'une webcam connectée au pc
///
/// Elle est prévue pour fonctionner dans un thread indépendant du thread principal.
/// Les échanges avec cette classe se font par le biais des signaux et slots qui sont thread-safe
///

class AbulEduCameraWorker : public QObject
{
    Q_OBJECT
    Q_FLAGS(ImageType ImageTypes)
public:
    enum ImageType
    {
        NoFlag          = 0x0,
        FullQImage      = 0x1, // QImage  complète
        FullCvMat       = 0x2, // cv::Mat complète
        CroppedImage    = 0x4, // QImage  zone sélectionnée ROI
        CroppedCvMat    = 0x8  // cv::Mat zone sélectionnée ROI
    };

    Q_DECLARE_FLAGS(ImageTypes, ImageType)
    explicit AbulEduCameraWorker(QObject *parent = 0);
    ~AbulEduCameraWorker();

private:
    /* capture une frame */
    void getFrame();

    bool m_localDebug;
    bool m_isCameraConnected;
    bool m_inited;
    int m_deviceNumber;
    int m_resWidth;
    int m_resHeight;
    cv::VideoCapture m_camera;
    cv::Mat m_frame;
    cv::Mat croppedMat;
    cv::Rect m_rectROI;

signals:
    ///
    /// \brief signalAbeCamWorkerIsCameraConnected passe true si la caméra est connectée
    ///
    void signalAbeCamWorkerIsCameraConnected(bool);

    ///
    /// \brief signalAbeCamWorkerImageCvMatReady passe la cv::Mat de l'image capturée
    ///
    void signalAbeCamWorkerImageCvMatReady(cv::Mat);

    ///
    /// \brief signalAbeCamWorkerCroppedImageCvMatReady passe la cv::Mat de la zone sélectionnée dans l'image capturée
    ///
    void signalAbeCamWorkerCroppedImageCvMatReady(cv::Mat);

    ///
    /// \brief signalAbeCamWorkerQImageReady passe la QImage de l'image capturée
    ///
    void signalAbeCamWorkerQImageReady(QImage);

    ///
    /// \brief signalAbeCamWorkerCroppedQImageReady passe la QImage de la zone sélectionnée dans l'image capturée
    ///
    void signalAbeCamWorkerCroppedQImageReady(QImage);

    ///
    /// \brief signalAbeCamWorkerImageSize passe les dimensions de l'image capturée
    ///
    void signalAbeCamWorkerImageSize(int, int);

    ///
    /// \brief signalAbeCamWorkerError retourne un numéro d'erreur
    ///
    ///  0 = Tout va bien, 1 = Camera non connectée
    ///
    void signalAbeCamWorkerError(int);

public slots:
    ///
    /// \brief slotAbeCamWorkerSetDevice initialise la camera passée en paramètre
    ///
    /// \note Par défaut, la caméra passée en paramètre est initialisée dans sa plus grande résolution.
    /// \param deviceNumber
    ///
    void slotAbeCamWorkerSetDevice(int deviceNumber, int width, int height);

    ///
    /// \brief slotAbeCamWorkerSetResolution passe la caméra dans la résolution demandée
    /// \param resWidth Nombre de pixels en largeur
    /// \param resHeight Nombre de pixels en hauteur
    ///
    void slotAbeCamWorkerSetResolution(int resWidth, int resHeight);

    ///
    /// \brief slotAbeCamWorkerGetFullImage capture une image sous forme de QImage
    ///
    /// \note le retour se fait par le signal signalAbeCamWorkerQImageReady(QImage)
    /// \note Bien que ce soit possible, on ne devrait pas utiliser directement cette fonction, mais plutôt :
    /// slotAbeCamWorkerGetFrame(AbulEduCameraWorker::ImageTypes imageTypes);
    ///
    void slotAbeCamWorkerGetFullImage();

    ///
    /// \brief slotAbeCamWorkerGetCroppedImage capture une image sous forme de QImage
    ///
    /// \note le retour se fait par le signal signalAbeCamWorkerCroppedQImageReady(QImage)
    /// \note Bien que ce soit possible, on ne devrait pas utiliser directement cette fonction, mais plutôt :
    /// slotAbeCamWorkerGetFrame(AbulEduCameraWorker::ImageTypes imageTypes);
    ///
    void slotAbeCamWorkerGetCroppedImage();

    ///
    /// \brief slotAbeCamWorkerGetFullCvMatImage capture une image sous forme de cv::Mat (opencv2)
    ///
    /// \note le retour se fait par le signal signalAbeCamWorkerImageCvMatReady(cv::Mat)
    /// \note Bien que ce soit possible, on ne devrait pas utiliser directement cette fonction, mais plutôt :
    /// slotAbeCamWorkerGetFrame(AbulEduCameraWorker::ImageTypes imageTypes);
    ///
    void slotAbeCamWorkerGetFullCvMatImage();

    ///
    /// \brief slotAbeCamWorkerGetCroppedCvMatImage capture une image sous forme de cv::Mat (opencv2)
    ///
    /// \note le retour se fait par le signal signalAbeCamWorkerCroppedImageCvMatReady(cv::Mat)
    /// \note Bien que ce soit possible, on ne devrait pas utiliser directement cette fonction, mais plutôt :
    /// slotAbeCamWorkerGetFrame(AbulEduCameraWorker::ImageTypes imageTypes);
    ///
    void slotAbeCamWorkerGetCroppedCvMatImage();

    ///
    /// \brief slotAbeCamWorkerGetFrame capture une image
    ///
    /// \note le retour se fait en fonction du paramètre passé FullQImage, FullCvMAt, CroppedQImage, CroppedCvMat
    /// ou une combinaison des quatre.
    /// \param imageTypes
    ///
    void slotAbeCamWorkerGetFrame(AbulEduCameraWorker::ImageTypes imageTypes);


    ///
    /// \brief slotAbeCamWorkerSetROI fixe la Region Of Interest de l'image (rectangle)
    /// \param le rectangle de sélection
    ///
    void slotAbeCamWorkerSetROI(QRect rect);

private slots:
    void initCapture();
};

Q_DECLARE_METATYPE(AbulEduCameraWorker::ImageTypes)

#endif // ABULEDUCAMERAWORKER_H

